//
//  ToyDetailCell.swift
//  SwiftDemo
//
//  Created by Lovense on 2024/7/2.
//

import UIKit

class ToyDetailCell: UITableViewCell {
    var slider: UISlider?
    var sliderValueLabel: UILabel?
    var btn: UIButton?
    lazy var inputTextField: UITextField = {
        let inputTextField = UITextField(frame: .init(x: 5, y: 10, width: UIScreen.main.bounds.size.width - 10, height: 30))
        inputTextField.borderStyle = .line
        inputTextField.layer.borderWidth = 1
        inputTextField.layer.borderColor = UIColor.black.cgColor
        self.contentView.addSubview(inputTextField)
        return inputTextField
    }()
    
    var input1: UITextField?
    var input2: UITextField?
    var input3: UITextField?
    var titleLabel: UILabel?
    
    var sliderChangeBlock: ((Int) -> Void)?
    var btnClickBlock: (() -> Void)?
    var sendCommandBlock: ((String, String, String) -> Void)?
    
    func setSliderName(_ name: String, min: Int = 0, max: Int = 20) {
        if let slider {
            self.textLabel?.text = name
        } else {
            self.textLabel?.numberOfLines = 0
            self.textLabel?.font = .systemFont(ofSize: 14)
            self.textLabel?.textAlignment = .center
            self.textLabel?.text = name
            let slider = UISlider(frame: .init(x: 15, y: 40, width: UIScreen.main.bounds.width - 30, height: 50))
            slider.minimumValue = Float(min)
            slider.maximumValue = Float(max)
            slider.value = 0
            slider.isContinuous = false
            slider.thumbTintColor = .blue
            slider.addTarget(self, action: #selector(self.sliderChange(_:)), for: .valueChanged)
            
            let minLabel = UILabel(frame: .init(x: 30, y: 20, width: 50, height: 20))
            minLabel.font = .systemFont(ofSize: 12)
            minLabel.text = min.description
            self.contentView.addSubview(minLabel)
            
            let sliderValueLabel = UILabel(frame: .init(x: UIScreen.main.bounds.size.width / 2 - 25, y: 20, width: 50, height: 20))
            sliderValueLabel.font = .boldSystemFont(ofSize: 13)
            sliderValueLabel.text = min.description
            sliderValueLabel.textColor = .systemBlue
            sliderValueLabel.textAlignment = .center
            self.contentView.addSubview(sliderValueLabel)
            self.sliderValueLabel = sliderValueLabel
            
            let maxLabel = UILabel(frame: .init(x: UIScreen.main.bounds.width - 30, y: 20, width: 30, height: 20))
            maxLabel.font = .systemFont(ofSize: 12)
            maxLabel.text = max.description
            self.contentView.addSubview(maxLabel)
            
            self.contentView.addSubview(slider)
            self.slider = slider
        }
    }
    func setBtnName(_ name: String) {
        if let btn {
            btn.setTitle(name, for: .normal)
        } else {
            let btn = UIButton(type: .system)
            btn.titleLabel?.numberOfLines = 2
            btn.titleLabel?.textAlignment = .center
            btn.setTitle(name, for: .normal)
            btn.frame = .init(x: 0, y: 10, width: UIScreen.main.bounds.size.width, height: 50)
            btn.addTarget(self, action: #selector(self.btnClick), for: .touchUpInside)
            self.contentView.addSubview(btn)
            self.btn = btn
        }
    }
    func setMultInput(title titleName: String, sender btnName: String) {
        if self.titleLabel == nil {
            let titleLabel = UILabel(frame: .init(x: 6, y: 6, width: UIScreen.main.bounds.width - 12, height: 30))
            titleLabel.numberOfLines = 0
            titleLabel.font = .systemFont(ofSize: 14)
            titleLabel.textAlignment = .center
            titleLabel.text = titleName
            self.contentView.addSubview(titleLabel)
            self.titleLabel = titleLabel
        }
        let btnWidth: CGFloat = 70
        if self.btn == nil {
            let btn = UIButton(type: .system)
            btn.backgroundColor = .gray
            btn.setTitle(btnName, for: .normal)
            btn.layer.cornerRadius = 3
            btn.layer.masksToBounds = true
            btn.setTitleColor(.white, for: .normal)
            btn.frame = .init(x: UIScreen.main.bounds.size.width - (btnWidth + 6), y: 42, width: btnWidth, height: 35)
            btn.addTarget(self, action: #selector(self.btnClick), for: .touchUpInside)
            self.contentView.addSubview(btn)
            self.btn = btn
        }
        let inputWidth = (UIScreen.main.bounds.size.width - (btnWidth + 12) - 18) / 3
        if self.input1 == nil {
            let input1 = UITextField(frame: .init(x: 6, y: 42, width: inputWidth, height: 35))
            input1.textAlignment = .center
            input1.borderStyle = .line
            input1.placeholder = "-1"
            input1.layer.cornerRadius = 3
            input1.layer.masksToBounds = true
            input1.layer.borderWidth = 1
            input1.layer.borderColor = UIColor.gray.cgColor
            self.contentView.addSubview(input1)
            self.input1 = input1
        }
        
        if self.input2 == nil {
            let input2 = UITextField(frame: .init(x: 6 + inputWidth + 6, y: 42, width: inputWidth, height: 35))
            input2.textAlignment = .center
            input2.borderStyle = .line
            input2.placeholder = "-1"
            input2.layer.cornerRadius = 3
            input2.layer.masksToBounds = true
            input2.layer.borderWidth = 1
            input2.layer.borderColor = UIColor.gray.cgColor
            self.contentView.addSubview(input2)
            self.input2 = input2
        }
        
        if self.input3 == nil {
            let input3 = UITextField(frame: .init(x: 6 + (inputWidth + 6) * 2, y: 42, width: inputWidth, height: 35))
            input3.textAlignment = .center
            input3.borderStyle = .line
            input3.placeholder = "-1"
            input3.layer.cornerRadius = 3
            input3.layer.masksToBounds = true
            input3.layer.borderWidth = 1
            input3.layer.borderColor = UIColor.gray.cgColor
            self.contentView.addSubview(input3)
            self.input3 = input3
        }
    }
    @objc func sliderChange(_ slider: UISlider) {
        self.sliderValueLabel?.text = Int(slider.value).description
        self.sliderChangeBlock?(Int(slider.value))
    }
    @objc func btnClick() {
        self.btnClickBlock?()
        
        if let sendCommandBlock {
            let input1 = {
                if let input1 = self.input1?.text, !input1.isEmpty {
                    return input1
                }
                return "-1"
            }()
            let input2 = {
                if let input2 = self.input2?.text, !input2.isEmpty {
                    return input2
                }
                return "-1"
            }()
            let input3 = {
                if let input3 = self.input3?.text, !input3.isEmpty {
                    return input3
                }
                return "-1"
            }()
            sendCommandBlock(input1, input2, input3)
        }
    }
}
